/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// in_revol.c -- input handler for the Nintendo Wii using devkitPPC / libogc
// (based on in_null.c)

#include <gccore.h>
#include <wiiuse/wpad.h>

#include "quakedef.h"

extern GXRModeObj* sys_rmode;

cvar_t m_filter = {"m_filter","0"};

int window_center_x;

int window_center_y;

u32 previous_key_status;

int previous_touch_x;

int previous_touch_y;

int old_mouse_x;

int old_mouse_y;

int mouse_x;

int mouse_y;

qboolean yaw_enabled;

float previous_cl_yaw;

float previous_yaw;

qboolean pitch_enabled;

float previous_cl_pitch;

float previous_pitch;

int wmote_pitch_len;

int wmote_pitch_cur;

float* wmote_pitch_hist;

float wmote_pitch_upperlimit;

float wmote_pitch_lowerlimit;

void IN_ResetWindowCenter()
{
	window_center_x = sys_rmode->viWidth / 2;
	window_center_y = sys_rmode->viHeight / 2;
}

void IN_Init (void)
{
	int i;

	IN_ResetWindowCenter();

	WPAD_ScanPads();
	previous_key_status = WPAD_ButtonsHeld(WPAD_CHAN_0);
	previous_touch_x = window_center_x;
	previous_touch_y = window_center_y;
	old_mouse_x = window_center_x;
	old_mouse_y = window_center_y;

	yaw_enabled = false;
	pitch_enabled = false;

	wmote_pitch_len = 6;
	wmote_pitch_hist = Sys_Malloc(sizeof(float) * wmote_pitch_len, "IN_Init");

	for(i = 0; i < wmote_pitch_len; i++)
		wmote_pitch_hist[i] = 0;
	
	wmote_pitch_cur = 0;

	wmote_pitch_upperlimit = 1;
	wmote_pitch_lowerlimit = -1;

	Cvar_RegisterVariable(&m_filter);
}

void IN_Shutdown (void)
{
}

void IN_Commands (void)
{
}

void IN_Move (usercmd_t *cmd)
{
	u32 k;
	ir_t p;
	orient_t o;
	expansion_t e;
	int x;
	int y;
	int mx;
	int my;
	int i;
	float f;
	float fp;

	WPAD_ScanPads();

	if(key_dest != key_console)
	{
		// Use the Wii Remote as a mouse substitute, but only if the console is not active: 

		k = WPAD_ButtonsHeld(WPAD_CHAN_0);

		// Do all of these only if the A button is pressed:
		if((k & WPAD_BUTTON_A) == WPAD_BUTTON_A)
		{

			WPAD_IR(WPAD_CHAN_0, &p);

			if(in_wlook)
			{
				// If in Wii Remote look mode, adjust yaw & pitch according to the orientation angles:

				WPAD_Orientation(WPAD_CHAN_0, &o);

				if(p.valid || (o.yaw != 0))
				{
					if(!yaw_enabled)
					{
						previous_cl_yaw = cl.viewangles[YAW];
						previous_yaw = o.yaw;
						yaw_enabled = true;
					};
				} else
				{
					yaw_enabled = false;
				};

				if(!pitch_enabled)
				{
					previous_cl_pitch = cl.viewangles[PITCH];
					previous_pitch = o.pitch;
					for(i = 0; i < wmote_pitch_len; i++)
						wmote_pitch_hist[i] = o.pitch;
					wmote_pitch_upperlimit = o.pitch + 1;
					wmote_pitch_lowerlimit = o.pitch - 1;
					pitch_enabled = true;
				};

				if(yaw_enabled)
				{
					cl.viewangles[YAW] = previous_cl_yaw + ((previous_yaw - o.yaw) * sensitivity.value);
					V_StopPitchDrift();
				};

				wmote_pitch_hist[wmote_pitch_cur] = o.pitch;

				f = 0;
				for(i = 0; i < wmote_pitch_len; i++)
					f += wmote_pitch_hist[i];
				f /= wmote_pitch_len;

				if(pitch_enabled)
				{
					if((f > wmote_pitch_upperlimit)||(f < wmote_pitch_lowerlimit))
					{
						fp = ((wmote_pitch_upperlimit + wmote_pitch_lowerlimit) / 2.0);

						cl.viewangles[PITCH] = previous_cl_pitch + ((fp - previous_pitch) * sensitivity.value);

						if (cl.viewangles[PITCH] > 80)
							cl.viewangles[PITCH] = 80;
						if (cl.viewangles[PITCH] < -70)
							cl.viewangles[PITCH] = -70;

					};
				};

				wmote_pitch_cur++;
				if(wmote_pitch_cur >= wmote_pitch_len)
					wmote_pitch_cur = 0;

				if(f > wmote_pitch_upperlimit)
				{
					wmote_pitch_lowerlimit += (f - wmote_pitch_upperlimit);
					wmote_pitch_upperlimit += (f - wmote_pitch_upperlimit);
				};

				if(f < wmote_pitch_lowerlimit)
				{
					wmote_pitch_upperlimit -= (wmote_pitch_lowerlimit - f);
					wmote_pitch_lowerlimit -= (wmote_pitch_lowerlimit - f);
				};

			} else
			{
				// If not in Wii Remote look mode, do Windows-style mouse handling:

				if(p.valid)
				{
					x = p.x;
					y = p.y;
				} else
				{
					x = previous_touch_x;
					y = previous_touch_y;
				};

				if((k & WPAD_BUTTON_A) != (previous_key_status & WPAD_BUTTON_A))
				{
					window_center_x = x;
					window_center_y = y;
				};

				mx = x - window_center_x;
				my = y - window_center_y;
				
				if (m_filter.value)
				{
					mouse_x = (mx + old_mouse_x) * 0.5;
					mouse_y = (my + old_mouse_y) * 0.5;
				}
				else
				{
					mouse_x = mx;
					mouse_y = my;
				};
				
				old_mouse_x = mx;
				old_mouse_y = my;
				
				mouse_x *= sensitivity.value;
				mouse_y *= sensitivity.value;

				if ( (in_strafe.state & 1) || (lookstrafe.value && (in_mlook.state & 1) ))
					cmd->sidemove += m_side.value * mouse_x;
				else
					cl.viewangles[YAW] -= m_yaw.value * mouse_x;

				if (in_mlook.state & 1)
					V_StopPitchDrift ();
					
				if ( (in_mlook.state & 1) && !(in_strafe.state & 1))
				{
					cl.viewangles[PITCH] += m_pitch.value * mouse_y;
					if (cl.viewangles[PITCH] > 80)
						cl.viewangles[PITCH] = 80;
					if (cl.viewangles[PITCH] < -70)
						cl.viewangles[PITCH] = -70;
				}
				else
				{
					if ((in_strafe.state & 1) && noclip_anglehack)
						cmd->upmove -= m_forward.value * mouse_y;
					else
						cmd->forwardmove -= m_forward.value * mouse_y;
				}

				previous_touch_x = window_center_x = x;
				previous_touch_y = window_center_y = y;

			};

		} else
		{

			yaw_enabled = false;
			pitch_enabled = false;

		};

		previous_key_status = k;

	};

	WPAD_Expansion(WPAD_CHAN_0, &e);
	
	cmd->sidemove += m_side.value * (e.nunchuk.js.pos.x - e.nunchuk.js.center.x) * sensitivity.value;
	cmd->forwardmove -= m_forward.value * (e.nunchuk.js.center.y - e.nunchuk.js.pos.y) * sensitivity.value;
	
}

