/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// sys_revol.c -- system driver for the Nintendo Wii using devkitPPC / libogc
// (based on sys_null.c)

#include <gccore.h>
#include <fat.h>
#include <unistd.h>
#include <wiiuse/wpad.h>
#include <ogc/lwp_watchdog.h>
#include <network.h>

#include "quakedef.h"
#include "errno.h"

qboolean isDedicated;

void* sys_framebuffer[2] = {NULL, NULL};

GXRModeObj* sys_rmode;

int sys_current_weapon;

extern qboolean	scr_drawdialog;

extern qboolean m_state_is_quit;

u32 sys_previous_keys;

double sys_previous_time;

int sys_netinit_error;

char sys_ipaddress_text[16];

int sys_current_character;

float sys_previous_char_time;

qboolean sys_current_char_count;

/*
===============================================================================

FILE IO

===============================================================================
*/

#define MAX_HANDLES             10
FILE    *sys_handles[MAX_HANDLES];

int             findhandle (void)
{
	int             i;
	
	for (i=1 ; i<MAX_HANDLES ; i++)
		if (!sys_handles[i])
			return i;
	Sys_Error ("out of handles");
	return -1;
}

/*
================
filelength
================
*/
int filelength (FILE *f)
{
	int             pos;
	int             end;

	pos = ftell (f);
	fseek (f, 0, SEEK_END);
	end = ftell (f);
	fseek (f, pos, SEEK_SET);

	return end;
}

int Sys_FileOpenRead (char *path, int *hndl)
{
	FILE    *f;
	int             i;
	
	i = findhandle ();

	f = fopen(path, "rb");
	if (!f)
	{
		*hndl = -1;
		return -1;
	}
	sys_handles[i] = f;
	*hndl = i;
	
	return filelength(f);
}

int Sys_FileOpenWrite (char *path)
{
	FILE    *f;
	int             i;
	
	i = findhandle ();

	f = fopen(path, "wb");
	if (!f)
		Sys_Error ("Error opening %s: %s", path,strerror(errno));
	sys_handles[i] = f;
	
	return i;
}

void Sys_FileClose (int handle)
{
	fclose (sys_handles[handle]);
	sys_handles[handle] = NULL;
}

void Sys_FileSeek (int handle, int position)
{
	fseek (sys_handles[handle], position, SEEK_SET);
}

void Sys_FileSeekCur (int handle, int offset)
{
	fseek (sys_handles[handle], offset, SEEK_CUR);
}

int Sys_FileRead (int handle, void *dest, int count)
{
	return fread (dest, 1, count, sys_handles[handle]);
}

int Sys_FileWrite (int handle, void *data, int count)
{
	return fwrite (data, 1, count, sys_handles[handle]);
}

int     Sys_FileTime (char *path)
{
	FILE    *f;
	
	f = fopen(path, "rb");
	if (f)
	{
		fclose(f);
		return 1;
	}
	
	return -1;
}

void Sys_mkdir (char *path)
{
}


/*
===============================================================================

SYSTEM IO

===============================================================================
*/

void Sys_MakeCodeWriteable (unsigned long startaddr, unsigned long length)
{
}


void Sys_Error (char *error, ...)
{
	va_list         argptr;

	printf ("Sys_Error: ");   
	va_start (argptr,error);
	vprintf (error,argptr);
	va_end (argptr);
	printf ("\n");

	VIDEO_SetNextFramebuffer(sys_framebuffer[1]);
	VIDEO_WaitVSync();

    Host_Shutdown();

	exit (1);
}

void Sys_Printf (char *fmt, ...)
{
	va_list         argptr;
	
	va_start (argptr,fmt);
	vprintf (fmt,argptr);
	va_end (argptr);
}

void Sys_Quit (void)
{
	vrect_t	vr;

	// These were copied from the quit message of the Windows version:
	M_DrawTextBox (0, 0, 38, 27);
	M_PrintWhite (16, 12,  "  Quake version 1.09 by id Software\n\n");
	M_PrintWhite (16, 28,  "Programming        Art \n");
	M_Print (16, 36,  " John Carmack       Adrian Carmack\n");
	M_Print (16, 44,  " Michael Abrash     Kevin Cloud\n");
	M_Print (16, 52,  " John Cash          Paul Steed\n");
	M_Print (16, 60,  " Dave 'Zoid' Kirsch\n");
	M_PrintWhite (16, 68,  "Design             Biz\n");
	M_Print (16, 76,  " John Romero        Jay Wilbur\n");
	M_Print (16, 84,  " Sandy Petersen     Mike Wilson\n");
	M_Print (16, 92,  " American McGee     Donna Jackson\n");
	M_Print (16, 100,  " Tim Willits        Todd Hollenshead\n");
	M_PrintWhite (16, 108, "Support            Projects\n");
	M_Print (16, 116, " Barrett Alexander  Shawn Green\n");
	M_PrintWhite (16, 124, "Sound Effects\n");
	M_Print (16, 132, " Trent Reznor and Nine Inch Nails\n\n");
	M_PrintWhite (16, 140, "Quake is a trademark of Id Software,\n");
	M_PrintWhite (16, 148, "inc., (c)1996 Id Software, inc. All\n");
	M_PrintWhite (16, 156, "rights reserved. NIN logo is a\n");
	M_PrintWhite (16, 164, "registered trademark licensed to\n");
	M_PrintWhite (16, 172, "Nothing Interactive, Inc. All rights\n");
	M_PrintWhite (16, 180, "reserved.\n");
	M_PrintWhite (16, 196, "Modifications for Nintendo Wii using\n");
	M_PrintWhite (16, 204, "devkitPPC / libogc:\n");
	M_PrintWhite (16, 212, "(c) 2009 Heriberto Delgado.");
	vr.x = vr.y = 0;
	vr.width = vid.width;
	vr.height = vid.height;
	vr.pnext = NULL;
	VID_Update (&vr);

    Host_Shutdown();

	exit (0);
}

double Sys_FloatTime (void)
{
	u64 t;
	u32 m;

	t = gettime();
	m = ticks_to_millisecs(t);
	return ((double)m) / 1000.0;
}

char *Sys_ConsoleInput (void)
{
	return NULL;
}

void Sys_Sleep (void)
{
}

qboolean Sys_KeyPressed(qboolean started)
{
	double t;

	if(started)
	{
		sys_previous_char_time = Sys_FloatTime();
		sys_current_char_count = 0;
		t = 0;
	} else
	{
		t = Sys_FloatTime() - sys_previous_char_time;
	};
	if((t == 0)&&(sys_current_char_count == 0))
	{
		sys_current_char_count++;
		return true;
	} else if((t > 0.4)&&(sys_current_char_count == 1))
	{
		sys_previous_char_time += 0.4;
		sys_current_char_count++;
		return true;
	} else if((t > 0.1)&&(sys_current_char_count > 1))
	{
		sys_previous_char_time += 0.1;
		sys_current_char_count++;
		return true;
	} else
	{
		return false;
	};
}

void Sys_SendKeyEvents (void)
{
	u32 k;

	WPAD_ScanPads();
	k = WPAD_ButtonsHeld(WPAD_CHAN_0);
	if((sys_previous_keys & WPAD_BUTTON_1) != (k & WPAD_BUTTON_1))
	{
		Key_Event(K_ENTER, ((k & WPAD_BUTTON_1) == WPAD_BUTTON_1));
	};
	if((sys_previous_keys & WPAD_BUTTON_2) != (k & WPAD_BUTTON_2))
	{
		Key_Event(K_ESCAPE, ((k & WPAD_BUTTON_2) == WPAD_BUTTON_2));
	};
	if((sys_previous_keys & WPAD_BUTTON_PLUS) != (k & WPAD_BUTTON_PLUS))
	{
		if(((k & WPAD_BUTTON_PLUS) == WPAD_BUTTON_PLUS)&&(sys_current_weapon < 8))
		{
			sys_current_weapon++;
		};
		Key_Event('0' + sys_current_weapon, ((k & WPAD_BUTTON_PLUS) == WPAD_BUTTON_PLUS));
	};
	if((sys_previous_keys & WPAD_BUTTON_MINUS) != (k & WPAD_BUTTON_MINUS))
	{
		if(((k & WPAD_BUTTON_MINUS) == WPAD_BUTTON_MINUS)&&(sys_current_weapon > 1))
		{
			sys_current_weapon--;
		};
		Key_Event('0' + sys_current_weapon, ((k & WPAD_BUTTON_MINUS) == WPAD_BUTTON_MINUS));
	};
	if((sys_previous_keys & WPAD_BUTTON_B) != (k & WPAD_BUTTON_B))
	{
		Key_Event(K_MOUSE1, ((k & WPAD_BUTTON_B) == WPAD_BUTTON_B));
	};
	if((sys_previous_keys & WPAD_NUNCHUK_BUTTON_Z) != (k & WPAD_NUNCHUK_BUTTON_Z))
	{
		Key_Event(' ', ((k & WPAD_NUNCHUK_BUTTON_Z) == WPAD_NUNCHUK_BUTTON_Z));
	};
	if((k & WPAD_NUNCHUK_BUTTON_C) != WPAD_NUNCHUK_BUTTON_C)
	{
		if((sys_previous_keys & WPAD_BUTTON_UP) != (k & WPAD_BUTTON_UP))
		{
			Key_Event(K_UPARROW, ((k & WPAD_BUTTON_UP) == WPAD_BUTTON_UP));
		};
		if((sys_previous_keys & WPAD_BUTTON_DOWN) != (k & WPAD_BUTTON_DOWN))
		{
			Key_Event(K_DOWNARROW, ((k & WPAD_BUTTON_DOWN) == WPAD_BUTTON_DOWN));
		};
		if((sys_previous_keys & WPAD_BUTTON_LEFT) != (k & WPAD_BUTTON_LEFT))
		{
			Key_Event(K_LEFTARROW, ((k & WPAD_BUTTON_LEFT) == WPAD_BUTTON_LEFT));
		};
		if((sys_previous_keys & WPAD_BUTTON_RIGHT) != (k & WPAD_BUTTON_RIGHT))
		{
			Key_Event(K_RIGHTARROW, ((k & WPAD_BUTTON_RIGHT) == WPAD_BUTTON_RIGHT));
		};
	};
	if(scr_drawdialog)
	{
		if((sys_previous_keys & WPAD_BUTTON_HOME) != (k & WPAD_BUTTON_HOME))
		{
			Key_Event('y', ((k & WPAD_BUTTON_HOME) == WPAD_BUTTON_HOME));
		};
	} else if(m_state_is_quit)
	{
		if((sys_previous_keys & WPAD_BUTTON_HOME) != (k & WPAD_BUTTON_HOME))
		{
			Key_Event('y', ((k & WPAD_BUTTON_HOME) == WPAD_BUTTON_HOME));
		};
	} else 
	{
		if((sys_previous_keys & WPAD_BUTTON_HOME) != (k & WPAD_BUTTON_HOME))
		{
			Key_Event(K_PAUSE, ((k & WPAD_BUTTON_HOME) == WPAD_BUTTON_HOME));
		};
		if((k & WPAD_NUNCHUK_BUTTON_C) == WPAD_NUNCHUK_BUTTON_C)
		{
			if((k & WPAD_BUTTON_UP) == WPAD_BUTTON_UP)
			{
				if(Sys_KeyPressed((sys_previous_keys & WPAD_BUTTON_UP) != (k & WPAD_BUTTON_UP)))
				{
					if(sys_current_character >= 126)
					{
						sys_current_character = 32;
					} else
					{
						sys_current_character++;
					};
					Key_Event(K_BACKSPACE, true);
					Key_Event(K_BACKSPACE, false);
					Key_Event(sys_current_character, true);
					Key_Event(sys_current_character, false);
				};
			};
			if((k & WPAD_BUTTON_DOWN) == WPAD_BUTTON_DOWN)
			{
				if(Sys_KeyPressed((sys_previous_keys & WPAD_BUTTON_DOWN) != (k & WPAD_BUTTON_DOWN)))
				{
					if(sys_current_character <= 32)
					{
						sys_current_character = 126;
					} else
					{
						sys_current_character--;
					};
					Key_Event(K_BACKSPACE, true);
					Key_Event(K_BACKSPACE, false);
					Key_Event(sys_current_character, true);
					Key_Event(sys_current_character, false);
				};
			};
			if((k & WPAD_BUTTON_LEFT) == WPAD_BUTTON_LEFT)
			{
				if(Sys_KeyPressed((sys_previous_keys & WPAD_BUTTON_LEFT) != (k & WPAD_BUTTON_LEFT)))
				{
					Key_Event(K_BACKSPACE, true);
					Key_Event(K_BACKSPACE, false);
				};
			};
			if((k & WPAD_BUTTON_RIGHT) == WPAD_BUTTON_RIGHT)
			{
				if(Sys_KeyPressed((sys_previous_keys & WPAD_BUTTON_RIGHT) != (k & WPAD_BUTTON_RIGHT)))
				{
					Key_Event(sys_current_character, true);
					Key_Event(sys_current_character, false);
				};
			};
		};
	};

	sys_previous_keys = k;

}

void Sys_HighFPPrecision (void)
{
}

void Sys_LowFPPrecision (void)
{
}

// >>> FIX: For Nintendo DS using devkitARM / libnds
// New function to help local -> heap allocation issues
void* Sys_Malloc(int size, char* purpose)
{
	void* m;
	
	m = malloc(size);
	if(m == 0)
	{
		Sys_Error ("Sys_Malloc: %s - failed on %i bytes", purpose, size);
	};
	return m;
}
// <<< FIX

//=============================================================================

void Sys_PowerOff(s32 chan)
{
	if(chan == WPAD_CHAN_0)
	{
		SYS_ResetSystem(SYS_POWEROFF, 0, 0);
	};
}

int main (int argc, char* argv[])
{
	isDedicated = false;

	sys_current_weapon = 1;

	sys_current_character = 'A';

	VIDEO_Init();
	WPAD_Init();

	sys_rmode = VIDEO_GetPreferredMode(NULL);
	VIDEO_Configure(sys_rmode);

	sys_framebuffer[0] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(sys_rmode));
	sys_framebuffer[1] = MEM_K0_TO_K1(SYS_AllocateFramebuffer(sys_rmode));

	VIDEO_ClearFrameBuffer(sys_rmode, sys_framebuffer[0], COLOR_BLACK);

	CON_Init(sys_framebuffer[1], 20, 20, sys_rmode->fbWidth, sys_rmode->xfbHeight, sys_rmode->fbWidth * VI_DISPLAY_PIX_SZ);

	VIDEO_SetNextFramebuffer(sys_framebuffer[0]);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(sys_rmode->viTVMode & VI_NON_INTERLACE)
	{
		VIDEO_WaitVSync();
	};

	WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);
	WPAD_SetVRes(WPAD_CHAN_0, sys_rmode->fbWidth, sys_rmode->xfbHeight);

	if(!fatInitDefault())
	{
		Sys_Error("Filesystem not enabled");
	};

	sys_previous_time = Sys_FloatTime();
	do 
	{
		sys_netinit_error = if_config(sys_ipaddress_text, NULL, NULL, true);
	} while((sys_netinit_error == -EAGAIN)&&((Sys_FloatTime() - sys_previous_time) < 3));
	if(sys_netinit_error < 0)
	{
		Sys_Printf("Network not enabled\n");
	};

	static quakeparms_t    parms;

	parms.memsize = 8*1024*1024;
	parms.membase = malloc (parms.memsize);
	parms.basedir = "fat:/";

	COM_InitArgv (argc, argv);

	parms.argc = com_argc;
	parms.argv = com_argv;

	printf ("Host_Init\n");
	Host_Init (&parms);

	// By default, enable Wii Remote look mode:
	IN_EnableWmoteLook();

	WPAD_SetPowerButtonCallback(Sys_PowerOff);

	while (1)
	{
		sys_previous_time = Sys_FloatTime();
		Host_Frame (1.0/30.0);
		VID_DrawWmoteGuide();
		VIDEO_WaitVSync();
		while((Sys_FloatTime() - sys_previous_time) < (1.0/30.0))
		{
			LWP_YieldThread();
		};
	}
}


